@@ -5,7 +5,7 @@ import torch.nn.functional as F
 import torch.autograd as autograd
 
 class MLP(nn.Module):
-    def __init__(self, d_in, d_hidden, d_out, activation=F.relu, bias=True, device="cpu"):
+    def __init__(self, d_in, d_hidden, d_out, activation=F.gelu, bias=True, device="cpu"):
         super().__init__()
         if d_hidden is None:
             d_hidden = d_in
