@@ -2,6 +2,8 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torch.linalg as LA
+from torch.nn.attention.flex_attention import flex_attention, create_block_mask
+# if torch.cuda.is_available(): flex_attention = torch.compile(flex_attention, backend="aot_eager")
 import math
 import warnings
 import opt_einsum
@@ -416,7 +418,7 @@ class SlidingWindowAttention(nn.Module):
     """
     def __init__(self, d_model, n_heads, window_len, dilation=1,
                  attn_sink=False, dropout=0.0, bias=True, batch_first=False,
-                 masked_window=True, device="cpu"):
+                 use_flex_attn=True, device="cpu"):
         super().__init__()
         self.d_model = d_model
         self.n_heads = n_heads
@@ -425,7 +427,7 @@ class SlidingWindowAttention(nn.Module):
         self.dropout = dropout
         self.batch_first = batch_first
         self.d_head = d_model // n_heads
-        self.masked_window = masked_window
+        self.use_flex_attn = use_flex_attn
         self.attn_sink = attn_sink
         self.device = device
         
@@ -437,7 +439,7 @@ class SlidingWindowAttention(nn.Module):
         self.out_proj = nn.Linear(d_model, d_model, bias=bias, device=device)
         
         self._reset_parameters()
-        
+    
     def _reset_parameters(self):
         nn.init.constant_(self.beta, 0.)
         nn.init.xavier_uniform_(self.q_proj.weight)
@@ -534,11 +536,39 @@ class SlidingWindowAttention(nn.Module):
         out2 = padded[:, H_neg : H_neg + L]
         return out2.movedim((0,1), dims)
     
+    @lru_cache(maxsize=2)
+    def causal_windowed_block_mask(self, src_len, tgt_len=None, device=None):
+        if tgt_len is None: tgt_len = src_len
+        if device is None: device = self.device
+        
+        half_span = self.dilation * (self.window_len - 1)
+        
+        def mask_mod(b, h, q_idx, kv_idx):
+            diff = q_idx - kv_idx
+            return (diff >= 0) & (diff <= half_span) & ((diff % self.dilation) == 0)
+
+        return create_block_mask(mask_mod, B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, device=self.device)
+    
+    @lru_cache(maxsize=2)
+    def symmetric_windowed_block_mask(self, src_len, tgt_len=None, device=None):
+        if tgt_len is None: tgt_len = src_len
+        if device is None: device = self.device
+        
+        half = self.window_len // 2
+        half_span = self.dilation * half
+
+        def mask_mod(b, h, q_idx, kv_idx):
+            diff = q_idx - kv_idx
+            return diff.abs().le(half_span) & ((diff % self.dilation) == 0)
+
+        return create_block_mask(mask_mod,  B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, device=device)
+    
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
             x = x.transpose(0, 1)
         
         src_len, bsz, d_model = x.shape
+        tgt_len = src_len
         
         q = self.q_proj(x)  # (src_len, batch_size, d_model)
         k = self.k_proj(x)  # (src_len, batch_size, d_model)
@@ -562,7 +592,7 @@ class SlidingWindowAttention(nn.Module):
         # beta = F.softplus(self.beta).reshape(self.n_heads, 1, 1).repeat(bsz, 1, 1)
         q = q / (math.sqrt(self.d_head) * beta)
         
-        if self.masked_window:
+        if not self.use_flex_attn:
             if causal:
                 attn_mask = self.causal_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, src_len, src_len)
             else:
@@ -583,51 +613,74 @@ class SlidingWindowAttention(nn.Module):
             # Convert attention weights to probabilities
             attn_output_weights = F.softmax(attn_output_weights, dim=-1)
             attn_output_weights = F.dropout(attn_output_weights, p=self.dropout, training=self.training)
-            
-            if self.attn_sink: attn_output_weights = attn_output_weights[..., :-1]
-            
+                        
             # Apply attention weights to values
-            attn_output = torch.bmm(attn_output_weights, v)  # (bsz * n_heads, src_len, d_head)
-        else:
-            if causal:
-                pad_amount = min(src_len-1, (self.window_len-1) * self.dilation)
-                pad = (0, pad_amount)
-            else:
-                pad_amount = self.dilation * min((src_len - 1) // self.dilation, self.window_len // 2)
-                pad = (pad_amount, pad_amount)
-
-            q = dilated_sliding_window(q, size=src_len, dim=1, stride=self.dilation, dilation=1, pad=pad)
-
-            attn_output_weights = torch.einsum('zbsd, zsd -> zbs', q, k)  # (bsz * n_heads, n_bands, src_len)
+            attn_output = torch.bmm(attn_output_weights, v)[:, :src_len]  # (bsz * n_heads, src_len, d_head)
             
-            shifts = torch.arange(-pad[0], pad[1]+1, self.dilation, device=self.device)
-            attn_output_weights = self.shift(attn_output_weights, shifts, shift_min_max=(shifts[0], shifts[-1]), dims=(1, 2), fill_value=float('-inf'))
+            attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads)
+        else:
+            attn_device = "cuda" if self.device == "cuda" else "cpu"
+            
+            q = q.to(attn_device).to(torch.float32)
+            k = k.to(attn_device).to(torch.float32)
+            v = v.to(attn_device).to(torch.float32)
+            
+            if causal:
+                block_mask = self.causal_windowed_block_mask(src_len, tgt_len, device=attn_device)
+            else:
+                block_mask = self.symmetric_windowed_block_mask(src_len, tgt_len, device=attn_device)
             
             if self.attn_sink:
-                sink_weight = torch.zeros((bsz * self.n_heads, 1, src_len), dtype=x.dtype, device=self.device)  # (bsz * n_heads, 1, src_len)
-                attn_output_weights = torch.cat([attn_output_weights, sink_weight], dim=1)  # (bsz * n_heads, n_bands+1, src_len)
+                k = torch.cat([k, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=k.dtype, device=k.device)], dim=1)
+                v = torch.cat([v, torch.zeros((bsz*self.n_heads, 1, self.d_head), dtype=v.dtype, device=v.device)], dim=1)
+                tgt_len += 1
             
-            # Convert attention weights to probabilities
-            attn_output_weights = F.softmax(attn_output_weights, dim=1)
-            attn_output_weights = F.dropout(attn_output_weights, p=self.dropout, training=self.training)
+            q = rearrange(q, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
+            k = rearrange(k, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
+            v = rearrange(v, '(b h) s d -> b h s d', h=self.n_heads).contiguous()
             
-            if self.attn_sink: attn_output_weights = attn_output_weights[:, :-1]
+            with torch._dynamo.disable():
+                attn_output = flex_attention(q, k, v, block_mask=block_mask)[:, :, :src_len]
             
-            attn_output_weights = self.shift(attn_output_weights, -shifts, shift_min_max=(-shifts[-1], -shifts[0]), dims=(1, 2), fill_value=0.0)
+            attn_output = rearrange(attn_output, 'b h s d -> s b (h d)', h=self.n_heads).contiguous().to(self.device)
             
-            # Apply attention weights to values
-            attn_output = attn_output_weights.unsqueeze(-1) * v.unsqueeze(1)
+            # if causal:
+            #     pad_amount = min(src_len-1, (self.window_len-1) * self.dilation)
+            #     pad = (0, pad_amount)
+            # else:
+            #     pad_amount = self.dilation * min((src_len - 1) // self.dilation, self.window_len // 2)
+            #     pad = (pad_amount, pad_amount)
+
+            # q = dilated_sliding_window(q, size=src_len, dim=1, stride=self.dilation, dilation=1, pad=pad)
+
+            # attn_output_weights = torch.einsum('zbsd, zsd -> zbs', q, k)  # (bsz * n_heads, n_bands, src_len)
             
-            attn_output = self.shift(attn_output, shifts, shift_min_max=(shifts[0], shifts[-1]), dims=(1, 2), fill_value=0.0)
+            # shifts = torch.arange(-pad[0], pad[1]+1, self.dilation, device=self.device)
+            # attn_output_weights = self.shift(attn_output_weights, shifts, shift_min_max=(shifts[0], shifts[-1]), dims=(1, 2), fill_value=float('-inf'))
             
-            attn_output = attn_output.sum(dim=1)
+            # if self.attn_sink:
+            #     sink_weight = torch.zeros((bsz * self.n_heads, 1, src_len), dtype=x.dtype, device=self.device)  # (bsz * n_heads, 1, src_len)
+            #     attn_output_weights = torch.cat([attn_output_weights, sink_weight], dim=1)  # (bsz * n_heads, n_bands+1, src_len)
+            
+            # # Convert attention weights to probabilities
+            # attn_output_weights = F.softmax(attn_output_weights, dim=1)
+            # attn_output_weights = F.dropout(attn_output_weights, p=self.dropout, training=self.training)
+            
+            # if self.attn_sink: attn_output_weights = attn_output_weights[:, :-1]
+            
+            # attn_output_weights = self.shift(attn_output_weights, -shifts, shift_min_max=(-shifts[-1], -shifts[0]), dims=(1, 2), fill_value=0.0)
+            
+            # # Apply attention weights to values
+            # attn_output = attn_output_weights.unsqueeze(-1) * v.unsqueeze(1)
+            
+            # attn_output = self.shift(attn_output, shifts, shift_min_max=(shifts[0], shifts[-1]), dims=(1, 2), fill_value=0.0)
+            
+            # attn_output = attn_output.sum(dim=1)
         
         # Apply final projection
-        attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads)
         attn_output = self.out_proj(attn_output)
         
         # Return in the correct format depending on batch_first
         if self.batch_first:
             return attn_output.transpose(0, 1)
         return attn_output
-
