@@ -310,7 +310,7 @@ def test_epoch(model, test_loader, loss_fn, log_fn=default_log_fn, data_fn=defau
         wandb.log(test_metrics.to_dict(prefix="test/"))
 
 def train(epochs, train_steps, benchmark_name, model, train_loader, optimizer, loss_fn, log_fn=default_log_fn, data_fn=default_data_fn,
-          scheduler=None, device="cpu",
+          scheduler=None, device="cpu", compile_backend="aot_eager",
           train_config=None, mixed_precision=False,
           checkpoint_dir="", model_name=None,
           val_loader=None, test_loader=None,
@@ -352,7 +352,7 @@ def train(epochs, train_steps, benchmark_name, model, train_loader, optimizer, l
             model = allocate_dynamic_memory(model, train_loader.batch_size, min_len, max_len, device)
         else:
             # Compile the model for faster training
-            model = compile_model(model, train_loader.dataset[0][0].shape, device)
+            model = compile_model(model, train_loader.dataset[0][0].shape, compile_backend, device)
         
         # Train loop
         if epochs is not None:
@@ -476,6 +476,11 @@ def train_from_config_file(yaml_path, loss_fn, log_fn=default_log_fn, data_fn=de
                     
                     `loss_backoff_type` (default: consecutive): Type of invalid loss backoff.
                     
+                    `compile_backend` (default: auto):
+                        Backend to use for compiling the model.
+                        If "auto" the following backends will be tried in order:
+                        ["inductor", "aot_eager", "eager"]
+                    
                     `load_checkpoint` (default: False): Whether to attempt loading model from checkpoint.
                     
                     `mixed_precision` (default: False): Whether to use mixed precision for training.
@@ -648,6 +653,7 @@ def train_from_config_file(yaml_path, loss_fn, log_fn=default_log_fn, data_fn=de
             model_args["device"] = device
             
             # Initialize model
+            compile_backend = general_config.get("compile_backend", "auto")
             model = initialize_model(model_name, **model_args)
             
             # Initialize optimizer
@@ -708,7 +714,7 @@ def train_from_config_file(yaml_path, loss_fn, log_fn=default_log_fn, data_fn=de
                 model=model, optimizer=optimizer, scheduler=scheduler,
                 loss_fn=experiment_loss_fn, log_fn=experiment_log_fn, data_fn=experiment_data_fn,
                 train_config=train_config, mixed_precision=mixed_precision,
-                checkpoint_dir=checkpoint_dir,
+                compile_backend=compile_backend, checkpoint_dir=checkpoint_dir,
                 train_loader=train_loader, val_loader=val_loader, test_loader=test_loader,
                 wandb_logging=wandb_logging, wandb_entity=wandb_entity, wandb_project=wandb_project,
                 grad_clip_norm=grad_clip_norm, accumulation_steps=accumulation_steps,
@@ -716,6 +722,8 @@ def train_from_config_file(yaml_path, loss_fn, log_fn=default_log_fn, data_fn=de
                 checkpoint_freq=checkpoint_freq, val_freq=val_freq, info_freq=info_freq,
                 device=device,
             )
+            
+            del general_config, model_config, optimizer_config, scheduler_config, train_config
         except KeyboardInterrupt:
             successful = False
             terminate = input('Terminate all experiments? (y/n): ').strip().lower()
@@ -727,7 +735,5 @@ def train_from_config_file(yaml_path, loss_fn, log_fn=default_log_fn, data_fn=de
         if successful:
             successful_count += 1
             print(f'\033[92mExperiment [{i + 1}/{len(experiments)}] completed successfully\033[0m\n')
-        
-        del general_config, model_config, optimizer_config, scheduler_config, train_config
     
     print(f'\033[1m{successful_count}/{len(experiments)} experiments completed successfully\033[0m')
\ No newline at end of file
